@@ -70,7 +70,7 @@ from .util import (
 from typing import (Any, BinaryIO, Callable, Dict, IO, Iterable, Iterator, List, NoReturn,
                     Sequence, TYPE_CHECKING, Tuple, Type, Union)
 
-from git.types import Commit_ish, PathLike, TBD
+from git.types import Commit_ish, PathLike
 
 if TYPE_CHECKING:
     from subprocess import Popen
@@ -127,30 +127,17 @@ class IndexFile(LazyMixin, git_diff.Diffable, Serializable):
 
     def _set_cache_(self, attr: str) -> None:
         if attr == "entries":
-            # read the current index
-            # try memory map for speed
-            lfd = LockedFD(self._file_path)
-            ok = False
             try:
-                fd = lfd.open(write=False, stream=False)
-                ok = True
+                fd = os.open(self._file_path, os.O_RDONLY)
             except OSError:
                 # in new repositories, there may be no index, which means we are empty
                 self.entries: Dict[Tuple[PathLike, StageType], IndexEntry] = {}
                 return None
-            finally:
-                if not ok:
-                    lfd.rollback()
             # END exception handling
 
             stream = file_contents_ro(fd, stream=True, allow_mmap=True)
 
-            try:
-                self._deserialize(stream)
-            finally:
-                lfd.rollback()
-                # The handles will be closed on destruction
-            # END read from default index on demand
+            self._deserialize(stream)
         else:
             super(IndexFile, self)._set_cache_(attr)
 
@@ -181,7 +168,7 @@ class IndexFile(LazyMixin, git_diff.Diffable, Serializable):
         self.version, self.entries, self._extension_data, _conten_sha = read_cache(stream)
         return self
 
-    def _entries_sorted(self) -> List[TBD]:
+    def _entries_sorted(self) -> List[IndexEntry]:
         """:return: list of entries, in a sorted fashion, first by path, then by stage"""
         return sorted(self.entries.values(), key=lambda e: (e.path, e.stage))
 
@@ -427,8 +414,8 @@ class IndexFile(LazyMixin, git_diff.Diffable, Serializable):
             # END path exception handling
         # END for each path
 
-    def _write_path_to_stdin(self, proc: 'Popen', filepath: PathLike, item: TBD, fmakeexc: Callable[..., GitError],
-                             fprogress: Callable[[PathLike, bool, TBD], None],
+    def _write_path_to_stdin(self, proc: 'Popen', filepath: PathLike, item: PathLike, fmakeexc: Callable[..., GitError],
+                             fprogress: Callable[[PathLike, bool, PathLike], None],
                              read_from_stdout: bool = True) -> Union[None, str]:
         """Write path to proc.stdin and make sure it processes the item, including progress.
 
@@ -492,12 +479,13 @@ class IndexFile(LazyMixin, git_diff.Diffable, Serializable):
             are at stage 3 will not have a stage 3 entry.
         """
         is_unmerged_blob = lambda t: t[0] != 0
-        path_map: Dict[PathLike, List[Tuple[TBD, Blob]]] = {}
+        path_map: Dict[PathLike, List[Tuple[StageType, Blob]]] = {}
         for stage, blob in self.iter_blobs(is_unmerged_blob):
             path_map.setdefault(blob.path, []).append((stage, blob))
         # END for each unmerged blob
         for line in path_map.values():
             line.sort()
+
         return path_map
 
     @ classmethod
@@ -1201,7 +1189,6 @@ class IndexFile(LazyMixin, git_diff.Diffable, Serializable):
             handle_stderr(proc, checked_out_files)
             return checked_out_files
         # END paths handling
-        assert "Should not reach this point"
 
     @ default_index
     def reset(self, commit: Union[Commit, 'Reference', str] = 'HEAD', working_tree: bool = False,
